<?
    /*
     * File: patch.php
     *
     * Description:
     *     This file contains several functions for updating and
     * listing patches in the database for projects.
     *
     * Creator: Tim Dwelle
     * Created: ??/??/????
     *
     * Change Log:
     *     Tim Dwelle <?@?.?> ??/??/????
     *         - Intial file creation
     *     Chris Ryan <chris@greatbridge.com> 10/11/2000
     *         - Modified to use some of our internal functions and global
     *           variables for database connection.
     *     Chris Ryan <chris@greatbridge.com> 10/23/2000
     *         - Added file comments
     *         - Added the use of pg_freeresult() after finishing with the
     *           use of queries
     *     Chris Ryan <chris@greatbridge.com> 10/26/2000
     *         - Change patch_list function to select all patches
     *           if statusid is empty
     *         - changed the type_id for patch status from 3 (generic statuses)
     *           to 9 (patch specific statuses)
     */

    /* patch_add(str $patch_name, str $version, str $description,
     *           str $userfile_name, path $userfile, int $categoryid,
     *           int $creatorid)
     *
     *  RETURNS:
     *   int patch_id
     *
     *  DESCRIPTION:
     *   patch_add() creates a new patch.
     *
     */
    function patch_add($project_id, $patch_name, $version, $description,
                       $userfile_name, $userfile, $categoryid, $created_by) {
        global $PATCH_FILE_ROOT;
        global $PATCH_URL_ROOT;
        global $PATCH_DEFAULT_STATUS;
        global $conn;

        // set up connection... prep data
        $patch_name = sqlText($patch_name);
        $ver = sqlText($version);
        $description = sqlText($description);
        $statusid = $PATCH_DEFAULT_STATUS;

        $project_name = getProjectName($project_id);

        // insert the basic data
        $query = "BEGIN;
        
                  INSERT INTO patch
                     (project_id,
                      patch_name,
                      assigned_to,
                      version,
                      description,
                      file_path,
                      url,
                      status_id,
                      category_id,
                      created_by)
                  VALUES
                     ($project_id,
                      $patch_name,
                      NULL,
                      $ver,
                      $description,
                      NULL,
                      NULL,
                      $statusid,
                      $categoryid,
                      $created_by)";

        $result = pg_exec($conn, $query);
        if (pg_cmdtuples($result) > 0) {
            # free the result set since it was successfull and we
            # no longer need it
            pg_freeresult($result);

            # get the patch_id for new patch
            $query = "SELECT currval('patch_patch_id_seq'::text)";

            $result = pg_exec($conn, $query);
            $patch_id = pg_result($result, 0, 0);
            pg_freeresult($result);

            // make sure directories are ok
            $path = $PATCH_FILE_ROOT."/". $project_name ."/";
            if (!file_exists($path)) {
                mkdir ($path, 0775);
            }
            $path = $path.$version."/";
            if (!file_exists($path)) {
                mkdir ($path, 0775);
            }
    
            // make sure filename is ok
            $i = 0;
            $filename = $userfile_name;
            while (file_exists($path.$filename)) {
                $filename = $userfile_name . $i;
                ++$i;
            }
    
            // copy file into directory
            $file_path = $path.$filename;
            copy($userfile, $file_path) or die("could not copy file");
    
    
            // we got the file where we want it...
            // now lets update our new record
            $url = $PATCH_URL_ROOT."/".$project_name."/".$version."/".$filename;
            $url = sqlText($url);
            $file_path = sqlText($path.$filename);
            
            $query = "INSERT INTO patch_history
                         (patch_id,
                          description,
                          created_by)
                      VALUES
                         ($patch_id,
                          'Patch added.',
                          $created_by); 
            
                      UPDATE patch
                      SET    file_path = $file_path,
                             url = $url
                      WHERE  patch_id = $patch_id";
                      
            $result = pg_exec($conn, $query);
            if (pg_cmdtuples($result) > 0) {
                # free the result since it was successful and we no
                # longer have a use for it
                pg_freeresult($result);

                # we're good... commit it all
                $query = "COMMIT";
                
                $result = pg_exec($conn, $query);
                pg_freeresult($result); 

                return $patch_id;
            }
        }
        return false;
    }


    /* patch_update(int $patch_id, int $assignedtoid, int $statusid,
     *              bool $file_updated, str $userfile_name, path $userfile,
     *              int $userid)
     *
     *  RETURNS:
     *   boolean.  true if successful; false otherwise.
     *
     *  DESCRIPTION:
     *   patch_update() modifies an existing patch.  This may be the
     *   patch info (like who it is assigned to or what its status is)
     *   or updates to the patch itself.
     *
     */
    function patch_update($patch_id, $assignedtoid, $statusid, $file_updated,
                          $userfile_name, $userfile, $userid) {
        global $conn;

        if (!$assignedtoid) {
            $assignedtoid = "NULL";
        }           

        $query = "BEGIN;
        
                  UPDATE patch
                  SET    assigned_to = $assignedtoid,
                         status_id = $statusid
                  WHERE  patch_id = $patch_id";

        $result = pg_exec($conn, $query);
        if (pg_cmdtuples($result) > 0) {
            pg_freeresult($result);

            if ($file_updated) {
                $query = "SELECT file_path
                          FROM   patch
                          WHERE  patch_id = $patch_id";
    
                $result = pg_exec($conn, $query);
                $file_path = pg_result($result, 0, 0);
                pg_freeresult($result);

                copy($userfile, $file_path) or die("could not copy file");
            }

            $query = "INSERT INTO patch_history
                        (patch_id,
                         description,
                         created_by)
                     VALUES
                        ($patch_id,
                         'Patch updated.',
                         $userid)"; 

            $result = pg_exec($conn, $query);
            if (pg_cmdtuples($result) > 0) {
                pg_freeresult($result);

                $query = "COMMIT";
                $result = pg_exec($conn, $query);
                pg_freeresult($result);                

                return true;
            }
        }

        return false;
    }


    /* patch_comment_add(int $patch_id, str $comment, int $created_by)
     *
     *  RETURNS:
     *   boolean.  true if successful; false otherwise.
     *
     *  DESCRIPTION:
     *   patch_comment_add adds a comment associated with a patch.
     *
     */
    function patch_comment_add($patch_id, $comment, $created_by) {
        global $conn;

        $comment = sqlText($comment);

        $query = "INSERT INTO patch_comment
                     (patch_id,
                      comment,
                      created_by)
                  VALUES
                     ($patch_id,
                      $comment,
                      $created_by)";

        $result = pg_exec($conn, $query);
        if (pg_cmdtuples($result) > 0) {
            pg_freeresult($result);
            return true;
        }

        return false;
    }


    /* patch_list(int $project_id, int $statusid)
     *
     *  RETURNS:
     *   array patches[patch["id,"name","assignedto[id,name,
     *              email]","version","description","file_path",
     *              "url","status[id,name]", "category[id,
     *              name]","created","creator[id,name,
     *              email]"]
     *
     *  DESCRIPTION:
     *   The patch_list() function looks up all pathes based on
     *   the patch's project_id and statusid.
     *
     */
    function patch_list($project_id, $statusid) {
        global $conn;

        $query   = "SELECT patch_id AS id,
                           patch_name AS name,
                           assignedto.member_id AS assignedto_id,
                           assignedto.member_name AS assignedto,
                           assignedto.email AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category patch_status,
                           category patch_category
                    WHERE  patch.project_id = $project_id ".
                    (!empty($statusid)?" AND    patch.status_id = $statusid ":"")
                    ." AND    patch.assigned_to = assignedto.member_id
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id
                    UNION SELECT patch_id AS id,
                           patch_name AS name,
                           NULL AS assignedto_id,
                           NULL AS assignedto,
                           NULL AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category as patch_status,
                           category as patch_category
                    WHERE  patch.project_id = $project_id ".
                    (!empty($statusid)?" AND    patch.status_id = $statusid ":"")
                    ." AND    patch.assigned_to = NULL
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id";

        $result = pg_exec($conn, $query);
        $rows = pg_numrows($result);
        for ($i=0; $i < $rows; $i++) {
            $patch = pg_fetch_array($result, $i);
    
            $assignedto["id"] = $patch["assignedto_id"];
            $assignedto["name"] = $patch["assignedto"];
            $assignedto["email"] = $patch["assignedto_email"];
            unset($patch["assignedto_id"]);
            unset($patch["assignedto_email"]);
            $patch["assignedto"] = $assignedto;
    
            $creator["id"] = $patch["created_by"];
            $creator["name"] = $patch["creator"];
            $creator["email"] = $patch["creator_email"];
            unset($patch["created_by"]);
            unset($patch["creator_email"]);
            $patch["creator"] = $creator;
    
            $status["id"] = $patch["statusid"];
            $status["name"] = $patch["status"];
            unset($patch["statusid"]);
            $patch["status"] = $status;     
    
            $category["id"] = $patch["categoryid"];
            $category["name"] = $patch["category"];
            unset($patch["categoryid"]);
            $patch["category"] = $category;     

            $patches[] = $patch;
        }
        pg_freeresult($result); 

        return $patches;
    }       


    /* patch_list_by_user(int $project_id, str $username)
     *
     *  RETURNS:
     *   array patches[patch["id,"name","assignedto[id,name,
     *              email]","version","description","file_path",
     *              "url","status[id,name]", "category[id,
     *              name]","created","creator[id,name,
     *              email]"]
     *
     *  DESCRIPTION:
     *   The patch_list_by_user() function looks up all pathes
     *   based on the patch's project_id and username (either
     *   creator or assignedto).
     *
     */
    function patch_list_by_user($project_id, $username) {
        global $conn;
        $username = sqlText($username);

        $query   = "SELECT patch_id AS id,
                           patch_name AS name,
                           assignedto.member_id AS assignedto_id,
                           assignedto.member_name AS assignedto,
                           assignedto.email AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category patch_status,
                           category patch_category
                    WHERE  patch.project_id = $project_id
                    AND    patch.assigned_to = assignedto.member_id
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id
                    AND    (creator.member_name = $username
                    OR     assignedto.member_name = $username)
                    UNION SELECT patch_id AS id,
                           patch_name AS name,
                           NULL AS assignedto_id,
                           NULL AS assignedto,
                           NULL AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category patch_status,
                           category patch_category
                    WHERE  patch.project_id = $project_id
                    AND    patch.assigned_to = NULL
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id
                    AND    creator.member_name = $username";

        $result = pg_exec($conn, $query);
        $rows = pg_numrows($result);
        for ($i=0; $i < $rows; $i++) {
            $patch = pg_fetch_array($result, $i);
    
            $assignedto["id"] = $patch["assignedto_id"];
            $assignedto["name"] = $patch["assignedto"];
            $assignedto["email"] = $patch["assignedto_email"];
            unset($patch["assignedto_id"]);
            unset($patch["assignedto_email"]);
            $patch["assignedto"] = $assignedto;
    
            $creator["id"] = $patch["created_by"];
            $creator["name"] = $patch["creator"];
            $creator["email"] = $patch["creator_email"];
            unset($patch["created_by"]);
            unset($patch["creator_email"]);
            $patch["creator"] = $creator;
    
            $status["id"] = $patch["statusid"];
            $status["name"] = $patch["status"];
            unset($patch["statusid"]);
            $patch["status"] = $status;     
    
            $category["id"] = $patch["categoryid"];
            $category["name"] = $patch["category"];
            unset($patch["categoryid"]);
            $patch["category"] = $category;     

            $patches[] = $patch;
        }
        pg_freeresult($result);
 
        return $patches;
    } 


    /* patch_fetch(int $patch_id)
     *
     *  RETURNS:
     *   array patch["id,"name","assignedto[id,name,email]",
     *              "version","description","file_path",
     *              "url","status[id,name]", "category[id,
     *              name]","created","creator[id,name,
     *              email]","history","comments"]
     *
     *  DESCRIPTION:
     *   The patch_fetch() function looks up a patch based on
     *   the patch's integer patch_id.  If a match is found,
     *   the function returns an array containing all the
     *   patch's data.
     *
     */
    function patch_fetch($patch_id) {
        global $conn;

        $query   = "SELECT patch_id AS id,
                           patch_name AS name,
                           assignedto.member_id AS assignedto_id,
                           assignedto.member_name AS assignedto,
                           assignedto.email AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category patch_status,
                           category patch_category
                    WHERE  patch.patch_id = $patch_id
                    AND    patch.assigned_to = assignedto.member_id
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id
                    UNION SELECT patch_id AS id,
                           patch_name AS name,
                           NULL AS assignedto_id,
                           NULL AS assignedto,
                           NULL AS assignedto_email,
                           version,
                           description,
                           file_path,
                           url,
                           patch_status.cat_id as statusid,
                           patch_status.cat_name AS status,
                           patch_category.cat_id as categoryid,
                           patch_category.cat_name as category,
                           patch.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch,
                           member assignedto,
                           member creator,
                           category patch_status,
                           category patch_category
                    WHERE  patch.patch_id = $patch_id
                    AND    patch.assigned_to = NULL
                    AND    patch.created_by = creator.member_id
                    AND    patch.status_id = patch_status.cat_id
                    AND    patch.category_id = patch_category.cat_id";


        $result = pg_exec($conn, $query);
        $patch = pg_fetch_array($result, 0);
        pg_freeresult($result);

        $assignedto["id"] = $patch["assignedto_id"];
        $assignedto["name"] = $patch["assignedto"];
        $assignedto["email"] = $patch["assignedto_email"];
        unset($patch["assignedto_id"]);
        unset($patch["assignedto_email"]);
        $patch["assignedto"] = $assignedto;

        $creator["id"] = $patch["created_by"];
        $creator["name"] = $patch["creator"];
        $creator["email"] = $patch["creator_email"];
        unset($patch["created_by"]);
        unset($patch["creator_email"]);
        $patch["creator"] = $creator;

        $status["id"] = $patch["statusid"];
        $status["name"] = $patch["status"];
        unset($patch["statusid"]);
        $patch["status"] = $status;     

        $category["id"] = $patch["categoryid"];
        $category["name"] = $patch["category"];
        unset($patch["categoryid"]);
        $patch["category"] = $category;     

        // get history
        $query   = "SELECT history_id AS id,
                           description,
                           patch_history.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch_history,
                           member creator
                    WHERE  patch_id = $patch_id
                    AND    creator.member_id = patch_history.created_by";

        $result = pg_exec($conn, $query);
        $rows = pg_numrows($result);
        for ($i=0; $i <$rows; $i++) {
            $rs = pg_fetch_array($result, $i);

            $creator["id"] = $rs["created_by"];
            $creator["name"] = $rs["creator"];
            $creator["email"] = $rs["creator_email"];
            unset($rs["created_by"]);
            unset($rs["creator_email"]);
            $rs["creator"] = $creator;

            $history[] = $rs;
        }
        pg_freeresult($result);

        $patch["history"] = $history;

        # get comments
        $query   = "SELECT comment_id AS id,
                           comment,
                           patch_comment.date_created AS created,
                           creator.member_id AS created_by,
                           creator.member_name AS creator,
                           creator.email AS creator_email
                    FROM   patch_comment,
                           member creator
                    WHERE  patch_id = $patch_id
                    AND    patch_comment.created_by = creator.member_id";


        $result = pg_exec($conn, $query);
        $rows = pg_numrows($result);
        for ($i=0; $i <$rows; $i++) {
            $rs = pg_fetch_array($result, $i);

            $creator["id"] = $rs["created_by"];
            $creator["name"] = $rs["creator"];
            $creator["email"] = $rs["creator_email"];
            unset($rs["created_by"]);
            unset($rs["creator_email"]);
            $rs["creator"] = $creator;

            $comments[] = $rs;
        }
        pg_freeresult($result);
        $patch["comments"] = $comments;

        return $patch;
    }       



    /* patch_statuses_fetch()
     *
     *  RETURNS:
     *   array statuses[status id=>status name]
     *
     *  DESCRIPTION:
     *   patch_statuses_fetch() function returns a list
     *   of all statuses that are in the database.
     *   The function returns an array of status ids
     *   mapped to status names.
     *
     */
    function patch_statuses_fetch() {
        global $conn;

        $query   = "SELECT    cat_id,
                              cat_name
                    FROM      category
                    WHERE     type_id = 9
                    ORDER BY  cat_id";

        $result = pg_exec($conn, $query);       
        $rows = pg_numrows($result);
        for ($i=0; $i < $rows; $i++) {
            $rs = pg_fetch_array($result,$i);
            $key = $rs["cat_id"];
            $val = $rs["cat_name"];
            $statuses[$key] = $val;
        }
        pg_freeresult($result);

        return $statuses;
    }


    /* patch_categories_fetch()
     *
     *  RETURNS:
     *   array categories[category id=>category name]
     *
     *  DESCRIPTION:
     *   patch_categories_fetch() function returns a list
     *   of all categories that are in the database.
     *   The function returns an array of category ids
     *   mapped to category names.
     *
     */
    function patch_categories_fetch() {
        global $conn;

        $query   = "SELECT    cat_id,
                              cat_name
                    FROM      category
                    WHERE     type_id = 4
                    ORDER BY  cat_id";

        $result = pg_exec($conn, $query);       
        $rows = pg_numrows($result);
        for ($i=0; $i < $rows; $i++) {
            $rs = pg_fetch_array($result,$i);
            $key = $rs["cat_id"];
            $val = $rs["cat_name"];
            $categories[$key] = $val;
        }
        pg_freeresult($result);

        return $categories;
    }
?>
