/*
 * File: member.sql
 *
 * Description:
 *     Contains the tables used for members, login and other misc. stuff
 *
 * Creator: Allen McGrath
 * Created: 09/18/2000
 *
 * Change log:
 *     Allen McGrath <amcgrath@greatbridge.com> 09/18/2000
 *         - Initial Creation
 *     Chris Ryan <chris@greatbridge.com> 09/18/2000
 *         - Made some format changes and added some references on some tables
 *         - Added table comments
 *     Chris Ryan <chris@greatbridge.com> 09/19/2000
 *         - Changed date fields to timestamp fields except for join_date on
 *           member table
 *         - Added the last_login field on member
 *     Chris Ryan <chris@greatbridge.com> 09/20/2000
 *         - Added the staff_email table that Allen McGrath wrote
 *         - Changed the reference clause on staff_email to point from
 *           feedback.cat_id to category.cat_id
 *         - Added group_id to category
 *     Chris Ryan <chris@greatbridge.com> 09/22/2000
 *         - Added register_id to the member table for Allen McGrath
 *         - Changed all SERIAL types to INT4's and manually created the
 *           sequences and set the defaults to nextval('<sequence_name>')
 *         - Added a confirmed field on member
 *     Chris Ryan <chris@greatbridge.com> 09/25/2000
 *         - Changed type_name on the type table from varchar(10) to varchar(50)
 *         - Added description on type of type text
 *         - Moved the following tables to the project.sql files:
 *               project, role and member_role
 *     Allen McGrath <allen@greatbridge.com> 09/25/2000
 *         - altered table MEMBER_PAGE, added TEMP_ID varchar(32)
 *         - this column will have a md5 sum isnerted via the web
 *         - so we can cut down on web-generated duplicate records.
 *     Chris Ryan <chris@greatbridge.com> 10/25/2000
 *         - Added cat_description field to category table
 *     Chris Ryan <chris@greatbridge.com> 10/30/2000
 *         - Changed all the text_x fields on member_page from varchar to text
 */

/*
 * Table: country
 *
 * Description:
 * This table is a list of all the countries wit a country code id.
 *
 * Field Description:
 * country_code		a unique code for the country
 * country_name		the name of this country
 */
CREATE TABLE country
(
    country_code  	VARCHAR(10) NOT NULL,
    country_name	VARCHAR(50) UNIQUE NOT NULL,
    PRIMARY KEY (country_code)
);

/*
 * Table: state
 *
 * Description:
 * The state table is a list of states related to a country
 *
 * Field Description:
 * state_code		the code of this state
 * state_name		the name of the state
 * country_code		the code of the country that this record is related to
 *
 * Foreign Keys:
 * country_code references country.country_code
 */
CREATE TABLE state
(
    state_code  	VARCHAR(4) NOT NULL,
    state_name		VARCHAR(50) UNIQUE NOT NULL,
    country_code	VARCHAR(10) NOT NULL REFERENCES country (country_code),
    primary key (state_code, country_code)
);

/*
 * Table: member
 *
 * Description:
 * The member table is the table that contains member information of people
 * who have registered on our site.
 *
 * Field Description:
 * member_id		the id of this member
 * member_name		the username this member has choosen
 * password		the password of this user... should be encrypted
 * hint			this is a hint that a user can use in case they forget
 *                          his/her password
 * email		the email of this member
 * status		the status of this member
 * first_name		the first name of this person
 * last_name		the last name of this person
 * addr_1		the first line of this persons address
 * addr_2		the second line of this persons address
 * city			the city name this person is in
 * zip_code		the zip code of this individual
 * state_code		the state of the country that this person is in
 * country_code		the country this person is in
 * area_code		the area code of the individuals phone number
 * phone		the phone number of this individual
 * register_id          the registration id of this member: used for email
 *                      verification and duplicate clicks on signup
 * confirmed            boolean if this user has confirmed his email/account
 * join_date		the date that this member joined
 * last_login		the date-time the member last logged in
 * last_updated		the last time this record was updated
 *
 * Foriegn Keys:
 * state_code references state.state_code
 * country_code references country.country_code
 */
CREATE SEQUENCE member_member_id_seq;
CREATE TABLE member 
(
    member_id		INT4 NOT NULL DEFAULT nextval('member_member_id_seq'), 
    member_name		VARCHAR(50) NOT NULL UNIQUE,
    password		VARCHAR(50) NOT NULL,
    hint		VARCHAR(150),
    email		VARCHAR(50) NOT NULL,
    status 		BOOL DEFAULT true,
    first_name		VARCHAR(100),
    last_name		VARCHAR(100),
    addr_1		VARCHAR(150),
    addr_2		VARCHAR(150),
    city		VARCHAR(50),
    zip_code		VARCHAR(15),
    state_code		VARCHAR(4) REFERENCES state (state_code),
    country_code	VARCHAR(10) REFERENCES country (country_code),
    area_code		VARCHAR(10),
    phone		VARCHAR(20),
    register_id		VARCHAR(32) not null UNIQUE,
    confirmed           BOOLEAN NOT NULL DEFAULT false,
    join_date		DATE DEFAULT NOW(),
    last_login		TIMESTAMP,
    last_updated	TIMESTAMP,
    primary key (member_id)
);
/*
 * Trigger: trigger_insert_update_member
 *
 * Description:
 * Whenever the member table is updated call this trigger to update the
 * last_updated field to the time the update happens. The function used in
 * the trigger is not located in this file but in the sites.sql file. The
 * function must exist for this to work properly
 */
CREATE TRIGGER trigger_insert_update_member
    BEFORE INSERT OR UPDATE ON member
    FOR EACH ROW EXECUTE PROCEDURE trigger_last_updated();

/*
 * Table: member_page
 *
 * Description:
 *
 * Field Description:
 * member_id		the member this record belongs to
 * heading_1
 * text_1
 * imagepage_1
 * heading_2
 * text_2
 * imagepage_2
 * heading_3
 * text_3
 * imagepage_3
 * heading_4
 * text_4
 * imagepage_4
 *
 * Foriegn Keys:
 * member_id references member.member_id on delete cascade
 */
CREATE TABLE member_page
(
    member_id		INT4 NOT NULL REFERENCES member (member_id)
                            ON DELETE CASCADE,
    heading_1		VARCHAR(50),
    text_1		text,
    imagepath_1		VARCHAR(100),
    heading_2		VARCHAR(50),
    text_2		text,
    imagepath_2		VARCHAR(100),
    heading_3		VARCHAR(50),
    text_3		text,
    imagepath_3		VARCHAR(100),
    heading_4		VARCHAR(50),
    text_4		text,
    imagepath_4		VARCHAR(100),
    temp_id		VARCHAR(32),
    primary key (member_id)
);

/*
 * Table: member_session
 *
 * Description:
 * This is the session table that stores the session_id of a member
 *
 * Field Description:
 * member_id		the id of the member for this session
 * session_id		the session id
 * last_updated		the last time this session was accessed
 *
 * Foreign Keys:
 * member_id references member.member_id on delete cascade
 */
CREATE TABLE member_session
(
    member_id		INT4 NOT NULL REFERENCES member (member_id)
                            ON DELETE CASCADE,
    session_id 		VARCHAR(32) NOT NULL,
    last_updated	TIMESTAMP DEFAULT NOW(),
    primary key (session_id)
);
/*
 * Trigger: trigger_access_member_session
 *
 * Description:
 * This trigger updates the last_updated field to the current time whenever
 * this record is created/updated/selected. The function used for this trigger
 * can be found in the sites.sql file. This function must be created before
 * creating this trigger or it won't work.
 */
CREATE TRIGGER trigger_insert_update_member_session
    BEFORE INSERT OR UPDATE ON member_session
    FOR EACH ROW EXECUTE PROCEDURE trigger_last_updated();

/*
 * Table: staff
 *
 * Description:
 * This is the staff list table of people who can be contacted for various
 * reasons.
 *
 * Field Description:
 * staff_id		the id of the staff member
 * email		the email address of the staff member
 * first_name		the first name of the staff member
 * last_name		the last name of the staff member
 * department		the department of the staff member
 */
CREATE SEQUENCE staff_staff_id_seq;
CREATE TABLE staff
(
    staff_id 		INT4 NOT NULL DEFAULT nextval('staff_staff_id_seq'),
    email		VARCHAR(50) NOT NULL,
    first_name		VARCHAR(100),
    last_name		VARCHAR(100),
    department		VARCHAR(100),
    primary key (staff_id)
);

/*
 * Table: type
 *
 * Description:
 * Different types for different things
 *
 * Field Description:
 * type_id		the id of this type
 * type_name		the name of this type
 */
CREATE SEQUENCE type_type_id_seq;
CREATE TABLE type
(
    type_id		INT4 NOT NULL UNIQUE DEFAULT nextval('type_type_id_seq'),
    type_name		VARCHAR(50),
    description		text,
    primary key (type_id)
);

/*
 * Table: category
 *
 * Description:
 * Different categories for different things
 *
 * Field Description:
 * cat_id		the id of this category
 * parent_id		the cat_id that is the parent of this category
 * type_id		the type of category this is
 * group_id		the site_group this category is part of
 * cat_name		the name of this category
 * cat_description      the description of this category
 *
 * Fopriegn Keys:
 * type_id references type.type_id
 */
CREATE SEQUENCE category_cat_id_seq;
CREATE TABLE category
(
    cat_id		INT4 NOT NULL UNIQUE DEFAULT nextval('category_cat_id_seq'),
    parent_id		INTEGER NOT NULL DEFAULT 0,
    type_id		INTEGER NOT NULL REFERENCES type (type_id),
    group_id            INTEGER NOT NULL REFERENCES site_group (group_id),
    cat_name		VARCHAR(50),
    cat_description     VARCHAR(128),
    primary key (cat_id)
);

/*
 * Table: feedback
 *
 * Description:
 * The feedback table stores feedback of users
 *
 * Field Description:
 * feedback_id		the feedback id passed from html form
 * member_id		the member who sumbitted the feedback, not mandatory
 * cat_id		the category that the feedback is for
 * email		the email address of the person submitting the feedback
 * comment		the persons feedback comment
 * sent_date		the date the feedback was sent
 * 
 * Foriegn Keys:
 * cat_id references category.cat_id
 */
CREATE TABLE feedback
(
    feedback_id		VARCHAR(32) NOT NULL,
    member_id 		INT4 NOT NULL default 0,
    cat_id		INT4 NOT NULL REFERENCES category (cat_id),
    email		VARCHAR(50) NOT NULL,
    comment		TEXT,
    sent_date		TIMESTAMP DEFAULT NOW(),
    primary key (feedback_id)
);

/*
 * Table: staff_email
 *
 * Description:
 * Grouping table for STAFF and FEEDBACK tables.
 *
 * Field Description:
 * staff_id     references staff.staff_id
 * cat_id       references category.cat_id
 */

create table staff_email (
    staff_id            INT4 NOT NULL REFERENCES staff(staff_id),
    cat_id              INT4 NOT NULL REFERENCES category(cat_id)
);

/*
 * Table: member_skill
 *
 * Description:
 * skills that members have.
 *
 * Field Description:
 */
CREATE TABLE member_skill (
    member_id      int4 not null references member (member_id),
    cat_id         int4 not null references category (cat_id),
    rating         int4 default 0,
    primary key (member_id,cat_id)
);


