/*
 * File: new_tables.sql
 *
 * Description:
 *     Contains the tables used for news.greatbridge.com, the marketing
 *     groups site
 *
 * Creator: Allen McGrath
 * Created: 09/29/2000
 *
 * Change log:
 *     Chris Ryan <chris@greatbridge.com> 11/01/2000
 *         - Changed the order in which the tables where created cause
 *           of the references dependency.
 *     Chris Ryan <chris@greatbridge.com> 11/15/2000
 *         - Added table mktg_contact
 *         - Added table mktg_response
 *         - Changed the type of num_emps from int4 to varchar(32) on the
 *           mktg_response table
 *     Chris Ryan <chris@greatbridge.com> 12/07/2000
 *         - Added column contact_me::bool to the mktg_response table
 */

/* 
 * Table: mktg_code
 * 
 * Description:
 * store all valid mktg codes marketing team comes up with
 * 
 *  Field Description:
 * mktg_code code entered by marketing team
 * create_date 	date record created
 * active  t= yes f=no
 * description description fo the code
*/
 CREATE TABLE mktg_code 
(
     mktg_code		varchar(20) not null unique,
     create_date	date default now(),
     active		bool default 't',
     description 	text,
     primary key (mktg_code)
);

/*
 * Table: news_access 
 *
 * Description:
 * stores all pages/files accessed on site by users
 * 
 * Field Description:
 * access_id		inserted via script (md5 generated).
 * create_date		date record was created.
 * mktg_code		code created via admin tool, found in mktg_code
 */
 CREATE TABLE news_access 
(
    access_id	  	VARCHAR(32) NOT NULL,
    create_date		DATE DEFAULT NOW(),
    mktg_code		varchar(25) not null references mktg_code,
    action		varchar(20),
    primary key (access_id,action)
);

/*
 * Table: mktg_contact
 *
 * Description:
 *     These are the records that have the contact information for are
 *     marketing mailing lists of people who visit the site from email/mail
 *     mass marketing campaigns
 *
 * Field Description:
 * id           an md5 sum value to identify the user (cookie set)
 * email        the email of the user
 * mktg_code    the marketing code
 * first_name   the first name of the user
 * last_name    the last_name of the user
 * company      the company of the user
 * address      the address of the user
 * city         the city
 * state        the state (this is the state.state_code selected)
 * zip          the zip code of the user
 * country      the country (this is the country.country_code selected)
 *
 * Primary Key  email + mktg_code
 */
CREATE TABLE mktg_contact (
    id          varchar(128) not null unique,
    email       varchar(512) not null,
    mktg_code   varchar(20),
    first_name  varchar(32) not null,
    last_name   varchar(32) not null,
    company     varchar(256),
    address     varchar(256),
    city        varchar(64),
    state       varchar(4),
    zip         varchar(20),
    country     varchar(4),
    primary key (email, mktg_code)
);

/*
 * Table: mktg_response
 *
 * Description:
 *     These are the survey responses of the users who come to the site from
 *     from marketing mass (e)mailings.
 *
 * Field Description:
 * id               an md5 sum value to identify the user: matches mktg_contact
 * os               the operating system they use
 * job              their primary job function
 * activity         the activity the perform in their job
 * num_emps         the number of employees at their company
 * using_pgsql      if the user uses pgsql
 * receive_monthly  if the user wants to receive monthly updates
 * mail_cd          if the user would like to get mailed new cd versions
 * comments         comments
 */
CREATE TABLE mktg_response (
    id              varchar(128) not null,
    os              varchar(128),
    job             varchar(128),
    activity        varchar(128),
    num_emps        varchar(32),
    using_pgsql     bool,
    receive_monthly bool,
    mail_cd         bool,
    comments        text,
    contact_me      bool,
    primary key (id)
);

